/****************************************************************************/
/*  Copyright 2005, Trustees of Indiana University                          */
/*  Author: Sidharth Thakur                                                 */
/*  Last modified: 14 January 2006                                          */
/*  License available at: <belt-trick-* folder>/docs/downloads/License.txt  */
/****************************************************************************/
#include "OGLWidgetFloating.h"
#include "VisibleForm.h"
#include <qmessagebox.h>
#include <qstring.h>

VisibleForm*	fwin_pForm;
char			fwin_qstr[128];		// String for onscreen text


void OGLWidgetFloating::text(char* string) {
  char* p;
  for(p=string;*p;p++) {
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12,*p);
  }
}


void OGLWidgetFloating::showOnScreenText() 
{
	double xpos=0.0;
	double ypos=0.0;
	double delta=0.07;
    
    glPushAttrib(GL_CURRENT_BIT | GL_LIGHTING_BIT);
    glDisable(GL_LIGHTING);
    
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
        glLoadIdentity();
        glMatrixMode(GL_PROJECTION);
        glPushMatrix();
        glLoadIdentity();
        gluOrtho2D(-1.2,1.2,-1.2,1.2);

        glMatrixMode(GL_MODELVIEW);
        glDisable(GL_LIGHTING);

        xpos=-1.0;
        ypos=-1.0;
        
		if(fwin_pForm->swapFrames) {
			glColor3f(0.0f,0.0f,0.7f);
			sprintf(fwin_qstr,"Quaternion Map of Frames on Belt:");
			glRasterPos2f(xpos,ypos);
			text(fwin_qstr);
			if(!hasFocus()) {
				xpos+=15.0*delta;
				glColor3f(0.7f,0.0f,0.0f);
				sprintf(fwin_qstr,"Click to focus...");
				glRasterPos2f(xpos,ypos);
				text(fwin_qstr);
			}
			xpos = -1.0;
			glColor3f(0.1f,0.1f,0.1f);
			ypos-=delta;
			switch(fwin_pForm->mo->qset->whichQuatComponent) {
				case SHOWXYZ:
					sprintf(fwin_qstr,"Components Projected = XYZ");
					break;
				case SHOWWYZ:
					sprintf(fwin_qstr,"Components Projected = WYZ");
					break;
				case SHOWWXZ:
					sprintf(fwin_qstr,"Components Projected = WXZ");
					break;
				case SHOWWXY:
					sprintf(fwin_qstr,"Components Projected = WXY");
					break;
				default:
					break;
			}
			glRasterPos2f(xpos,ypos);
			text(fwin_qstr);
		}
		else {
			ypos-=2.0*delta;
			//xpos+=3.5*delta;
			glColor3f(0.0f,0.0f,0.7f);
			sprintf(fwin_qstr,"Quaternion Map");
			glRasterPos2f(xpos,ypos);
			text(fwin_qstr);
			
			if(!hasFocus()) {
				xpos+=16.0*delta;
				glColor3f(0.7f,0.0f,0.0f);
				sprintf(fwin_qstr,"Click to focus...");
				glRasterPos2f(xpos,ypos);
				text(fwin_qstr);
			}
		}

        glMatrixMode(GL_PROJECTION);   
        glPopMatrix();
        glMatrixMode(GL_MODELVIEW);
    glPopMatrix();
    glPopAttrib();
}

OGLWidgetFloating::OGLWidgetFloating( QWidget* parent, const char* name, QWidget* parentForm)
: QGLWidget( parent, name)
{
	fwin_pForm = (VisibleForm*) parentForm;
    init();
}

OGLWidgetFloating::~OGLWidgetFloating() 
{
	fwin_pForm = NULL;
}

void OGLWidgetFloating::initializeGL () 
{
    glClearColor(0.975,0.975,0.975,1.0);
    //glClearColor(0.0,0.0,0.0,1.0);
    setupLights();
    m_viewScale = 1.0;
    
}

void OGLWidgetFloating::init ( void )
{
	// workspace center
    X=0; Y=0; dX=0; dY=0;
    XC = 0.0; YC = 0.0; ZC = -10.0;
    Cx = 0.0; Cy = 0.0; Cz = 0.0;

    // mouse stuff
	mouseButtons	= 0;
    button			= 0;
    mouseModifiers	= 0;

	// viewport dimensions
    xMin			= -1.0;
    xMax			=  1.0;
    yMin			= -1.0;
    yMax			=  1.0;
    zNear			=  5.0;
    zFar			= 25.0;

	main_height		= this->height();
	main_width		= this->width();

    /* local declarations */
    fwin_showAxes = 0;
    
    
}

void OGLWidgetFloating::setupLights ( void ) 
{
    static const GLfloat light_model_ambient_fwin[]	= {0.75f, 0.75f, 0.75f, 1.0f};
    static const GLfloat light0_ambient_fwin[]		= {0.75f, 0.75f, 0.75f, 1.0f};
    static const GLfloat light0_diffuse_fwin[]		= {1.0f, 1.0f, 1.0f, 1.0f};  
    static const GLfloat light0_specular_fwin[]		= {1.0f, 1.0f, 1.0f, 1.0f};
    static const GLfloat light0_shininess_fwin[]	= {500.0};
    static const GLfloat light0_direction_fwin[]	= {0.0f, 1.0f, 0.0f, 0.0f};    
    
    glDepthFunc(GL_LEQUAL);
    glEnable(GL_DEPTH_TEST);
    
    glEnable(GL_LIGHTING);
    glEnable(GL_NORMALIZE);
    glShadeModel(GL_SMOOTH);
    
    glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_FALSE);
    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);    
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, light_model_ambient_fwin);

    glLightfv(GL_LIGHT0, GL_DIFFUSE,   light0_diffuse_fwin);
    glLightfv(GL_LIGHT0, GL_POSITION,  light0_direction_fwin);
    glLightfv(GL_LIGHT0, GL_AMBIENT,   light0_ambient_fwin);
    glLightfv(GL_LIGHT0, GL_SPECULAR,  light0_specular_fwin);
    glLightfv(GL_LIGHT0, GL_SHININESS, light0_shininess_fwin);
    glEnable(GL_LIGHT0);     
	
    glEnable(GL_LIGHT0);

    ///* OpenGL Fog */
    //GLfloat fogColor[] = {0.3,0.3,0.3,1};   // fog color

    //// execute OpenGL commands
    //glEnable (GL_FOG);   // turn on fog, otherwise you won't see any

    //glFogi (GL_FOG_MODE, GL_EXP2);      // Fog fade using exponential function
    //glFogfv (GL_FOG_COLOR, fogColor);   // Set the fog color
    //glFogf (GL_FOG_DENSITY, 0.095);    // Set the density, don't make it too high.
    //glHint (GL_FOG_HINT, GL_DONT_CARE);    // ROS says Set default calculation mode
    //glFogf (GL_FOG_START, 0.0);
    //glFogf (GL_FOG_END, 1.0);

}


void OGLWidgetFloating::paintGL ()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	double factor = 25.0;
	double factor2= 3.5;

	glPushAttrib( GL_ALL_ATTRIB_BITS );
		glPushMatrix();
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		glEnable(GL_NORMALIZE);
		glDisable(GL_LIGHTING);
		glEnable(GL_COLOR_MATERIAL);
		//glDisable(GL_CULL_FACE);
		
		glBegin(GL_QUADS); //BACK
			glColor3f(0.9,0.9,0.9);
			glNormal3f(0.0f,0.0f,1.0f);
			glVertex3f(-factor2, -factor2, -factor);
			glVertex3f(+factor2, -factor2, -factor);
			glVertex3f(+factor2, +factor2, -factor);
			glVertex3f(-factor2, +factor2, -factor);
		glEnd();
		glBegin(GL_QUADS); //LEFT
			glColor3f(0.75,0.75,0.75);
			glNormal3f(1.0f,0.0f,0.0f);
			glVertex3f(-factor2, +factor2, -factor);
			glVertex3f(-factor2, +factor2, +factor);
			glVertex3f(-factor2, -factor2, +factor);
			glVertex3f(-factor2, -factor2, -factor);
		glEnd();
		glBegin(GL_QUADS); //RIGHT
			glColor3f(0.75,0.75,0.75);
			glNormal3f(-1.0f,0.0f,0.0f);
			glVertex3f(+factor2, +factor2, -factor);
			glVertex3f(+factor2, -factor2, -factor);
			glVertex3f(+factor2, -factor2, +factor);
			glVertex3f(+factor2, +factor2, +factor);
		glEnd();
		glBegin(GL_QUADS); //TOP
			glColor3f(0.6,0.6,0.6);
			glNormal3f(0.0f,1.0f,0.0f);
			glVertex3f(+factor2, +factor2, +factor);
			glVertex3f(+factor2, +factor2, -factor);
			glVertex3f(-factor2, +factor2, -factor);
			glVertex3f(-factor2, +factor2, +factor);
		glEnd();
		glPopMatrix();
	glPopAttrib();

	glPushMatrix();
		glTranslatef(Cx, Cy, 0);
		glScalef(m_viewScale, m_viewScale, m_viewScale);
		fwin_pForm->mo->showQuaternionMap();
	glPopMatrix();
	showOnScreenText();
}

void OGLWidgetFloating::resetApp ( void )
{
	m_viewScale = 1.0;

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glFrustum(xMin, xMax, yMin, yMax, zNear, zFar);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    
	//@ scale factor
	glScalef(m_viewScale, m_viewScale, m_viewScale);
    glTranslatef(XC,YC,ZC);
}

void  OGLWidgetFloating::resizeGL( int wd, int ht )
{
    int w = wd;
    int h = ht;

    glViewport(0,0,w,h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    glFrustum(xMin, xMax, yMin, yMax, zNear, zFar);
    /* Viewing location of camera */
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glTranslatef(XC,YC,ZC);
    
}

void OGLWidgetFloating::mousePressEvent ( QMouseEvent *e )
{
    QMouseEvent *m = e;
    QPixmap pixmap;
    int x = m->x();
    int y = m->y();

    setFocus();
    grabKeyboard();
    
    X = x;
    Y =  main_height -y - 1;

    switch (m->button()){
        case  LeftButton:
            button = 0x1; /* our Left   bit code */
            fwin_pForm->lb_status->setText(QString("Rotate model: Drag mouse while pressing Left-button"));
            pixmap = QPixmap::fromMimeSource("images/icon-action-drag-rotate.png");
            fwin_pForm->pix_status->setPixmap(pixmap);

            break;
        case MidButton:
            button = 0x2; /* our Middle bit code */
            fwin_pForm->lb_status->setText(QString("Slide Tracker: Drag mouse while pressing Middle-button"));
            pixmap = QPixmap::fromMimeSource("images/icon-action-drag-tracker.png");
            fwin_pForm->pix_status->setPixmap(pixmap);
            break;
        case RightButton:
            button = 0x4; /* our Right  bit code */
            fwin_pForm->lb_status->setText(QString("Zoom model: Drag mouse while pressing Right-button"));
            pixmap = QPixmap::fromMimeSource("images/icon-action-drag-zoom.png");
            fwin_pForm->pix_status->setPixmap(pixmap);

            break;
        default:
            break;
    }

    mouseButtons |= button;
    
    //Check if a vertex is lit
    if(button == 0x1){
        if(litVertex == -1){
			//if(draw_curve==OBJSPLINE) {
   //             //if not, add a new vertex
   //             printf("mouse clicked at (%d,%d)\n",X,Y);
   //             addPoint(X,Y);
   //         }
        }
        else{
			//printf("litVertex = %d\n",litVertex);
            movingPoint = TRUE;
        }   
    }

    updateGL();
}

void OGLWidgetFloating::mouseReleaseEvent ( QMouseEvent *e )
{
	QMouseEvent *evnt = e;
    QPixmap pixmap;
	mouseButtons &= (~(button));
    if(button == 0x1){
        movingPoint = FALSE;
    }
    fwin_pForm->lb_status->setText(QString(""));
    pixmap = QPixmap::fromMimeSource("images/icon-empty.png");
    fwin_pForm->pix_status->setPixmap(pixmap);

	evnt = NULL;
}

void OGLWidgetFloating::moveTransformedCoordinate(float x, float y)
{

    GLint viewport[4],winHeight;
    GLdouble mvmatrix[16], projmatrix[16];
    GLdouble wx,wy,wz, objx, objy, objz;
    
    
    glGetDoublev(GL_MODELVIEW_MATRIX, mvmatrix);
    glGetDoublev(GL_PROJECTION_MATRIX, projmatrix);
    glGetIntegerv(GL_VIEWPORT,viewport);
    winHeight = viewport[3];
    
    if(gluProject(xCoord[litVertex], yCoord[litVertex], zCoord[litVertex],
                  mvmatrix, projmatrix, viewport, &wx,&wy,&wz) == GL_FALSE){
        printf("gluProject() didn't work in moveTransformedCooordinates\n");
    }
    
    if(gluUnProject((GLdouble)x, 
                    (GLdouble)y,wz,mvmatrix,projmatrix,viewport, &objx,&objy,&objz) == GL_FALSE){
        printf("gluUnProject() didn't work in moveTransformedCooordinates\n");
    }
    
    xCoord[litVertex]=objx;
    yCoord[litVertex]=objy;
	//buzzform();
}
void OGLWidgetFloating::mouseMoveEvent ( QMouseEvent *e )
{
    QMouseEvent *m = e;
    int		x = m->x();
    int		y = m->y();
    double	deltaY	= 0.0;
    double	deltaX	= 0.0;
    int		nHatX	= 0;
    int		nHatY	= 0;
    double	thetaRad = 0.0;
    double	thetaDeg = 0.0;
    double	radius	 = 150.0;
    double	drSquared= 0.0;
    float	modelviewMatrix[16];
    int		deltaX_trackerpos = 0;
	int		scale_tracker = fwin_pForm->mo->bspheres/75;
	if(scale_tracker <= 1) 
		scale_tracker = 1;

    //@ scaling
    double	y1=0.0;
	double	y2=0.0;
    //@@
    
    xBegin = X;
    yBegin = Y;
    startY = Y;
    X = x;
    Y =  main_height - y - 1;	/* Invert to right-handed system */

    //@ scaling objects
    y1 = (double)yBegin;
    y2 = (double)Y;
    //@@

    xNext = X;
    yNext = Y;

   // Passive motion callback is simulate here with the
   // boolean movingPoint
    //if((enable_picking) && (draw_curve==OBJSPLINE) && (!movingPoint)) {
    //    mousePickVertex(x,y);
    //}
    
    //case 0x1:  /* Middle Mouse Button   */
    if(m->state()==MidButton) {
        
		
			fwin_pForm->mo->flag_do_not_compute = 1;
			//moveTransformedCoordinate(X,Y);
			deltaX_trackerpos = xNext - xBegin;
			if(deltaX_trackerpos > 1)
				deltaX_trackerpos = 2;
			if(deltaX_trackerpos < -1)
				deltaX_trackerpos = -2;

			fwin_pForm->mo->btrackerpos += scale_tracker*deltaX_trackerpos;
			if(fwin_pForm->mo->btrackerpos <0)
				fwin_pForm->mo->btrackerpos = 0;
			else if( fwin_pForm->mo->btrackerpos > fwin_pForm->mo->bspheres-1)
				fwin_pForm->mo->btrackerpos = fwin_pForm->mo->bspheres-1;

			fwin_pForm->notifyOGLWidgets();
		
    }
    
    //case 0x2:  /* Left Mouse Button - ROTATE OBJECT */
    if(m->state()==LeftButton){

            deltaX = xNext - xBegin;
            deltaY = yNext - yBegin;
            nHatX = (int)-deltaY;
            nHatY = (int)deltaX;
            //get the amount of rotation
            drSquared = pow(deltaX,2) + pow(deltaY,2);
            thetaRad = acos(radius/(sqrt(pow(radius,2) + drSquared)));
            thetaDeg = (thetaRad * 180) / M_PI;
            //translate to center
            glGetFloatv(GL_MODELVIEW_MATRIX,modelviewMatrix);
            glMatrixMode(GL_MODELVIEW);
            glLoadIdentity();
            glTranslatef(XC,YC,ZC);
            glTranslatef(Cx, Cy, 0);
            
            glRotatef(thetaDeg,nHatX,nHatY,0);
            //tranlsate back
            glTranslatef(-Cx, -Cy, 0);
            glTranslatef(-XC,-YC,-ZC);
            glMultMatrixf(modelviewMatrix);

        
    }
    
    //case 0x4:  /* Right Mouse Button  */
    //translating
    if( (m->state() & RightButton) && (m->state() & ShiftButton)) {
        
            //glGetFloatv(GL_MODELVIEW_MATRIX,modelviewMatrix);
            //glMatrixMode(GL_MODELVIEW);
            //glLoadIdentity();
            //glTranslatef(((float)xNext-xBegin)/((float)main_width/2.0),
            //((float)yNext-yBegin)/((float)main_height/2.0),0);
            //glMultMatrixf(modelviewMatrix);

        
        Cx = Cx + ((float)xNext-xBegin)/((float)main_width/2.0);
        Cy = Cy + ((float)yNext-yBegin)/((float)main_height/2.0);
    }
    else  if( (m->state() & RightButton) && !(m->state() & ShiftButton))//scaling
    {
        m_viewScale *= 1 + (y2 - y1)/((double)main_height*10.0);
        if(m_viewScale<0.05)
            m_viewScale=0.05;
        if(m_viewScale>5.0)
            m_viewScale=5.0;
    }        

	updateGL();
}

void OGLWidgetFloating::renderAxes ( void )
{
    double len = 0.5;
    double rad = 0.025;
    
    GLUquadricObj *qobj2;
    qobj2 = gluNewQuadric();

    glutSolidSphere(0.26,16,16);
    
    glPushAttrib(GL_CURRENT_BIT | GL_LIGHTING_BIT);
		glDisable(GL_LIGHTING);
		glPushMatrix();
			glTranslatef(0.0,0.0,0.0);
			glColor3f(0.0, 0.0, 1.0);
			glRotatef(-90,1,0,0);
			gluQuadricDrawStyle(qobj2, GLU_FILL);
			gluCylinder(qobj2, rad, rad, len, 16, 16);
		glPopMatrix();
		glPushMatrix();
			glTranslatef(0.0,0.0,0.0);
			glColor3f(0.0, 1.0, 0.0);
			glRotatef(90,0,1,0);
			gluQuadricDrawStyle(qobj2, GLU_FILL);
			gluCylinder(qobj2, rad, rad, len, 16, 16);
		glPopMatrix();
		glPushMatrix();
			glTranslatef(0.0,0.0,0.0);
			glColor3f(1.0, 0.0, 0.0);
			glRotatef(0,0,0,1);
			gluQuadricDrawStyle(qobj2, GLU_FILL);
			gluCylinder(qobj2, rad, rad, len, 16, 16);
		glPopMatrix();
    glPopAttrib();
    
}
void OGLWidgetFloating::keyPressEvent ( QKeyEvent *e )
{
	QMessageBox mb( "Exiting...",
                    "Really quit?",
                    QMessageBox::NoIcon,
                    QMessageBox::Yes | QMessageBox::Default,
                    QMessageBox::No,
					QMessageBox::NoButton );
    //modkey = e->key();
    switch(e->key()) {

        //CLEAR ALL
        case Key_Space:
            //myform->clearAll();
            break;
        //HELP    
        case Key_H:
            //this->releaseKeyboard();
            //i=QMessageBox::information(this, "Help", helpstr, QMessageBox::Ok);
            //if(i)
            //    this->grabKeyboard();
            break;
        //AXES
        case Key_A:
            fwin_pForm->mo->qshowAxes = !fwin_pForm->mo->qshowAxes;
            break;
        //B-SPLINE
        case Key_B:
            //currentSpline = BSPLINE;
            //whichSpline = BSPLINE;
            break;
        //CATMULL-ROM SPLINE    
        case Key_C:
            //currentSpline=CATROM;
            //whichSpline = CATROM;
            break;
        //EDGES
        case Key_E:
                /*
            pForm->mo->showEdgeOnly = !pForm->mo->showEdgeOnly;
            if(pForm->mo->showEdgeOnly == 1)
                pForm->cb_edge->setChecked(TRUE);
            else
                pForm->cb_edge->setChecked(FALSE);
                */
            break;
        //FRAME
        case Key_F:
                /*
            if(e->state() && ShiftButton) {
                pForm->mo->showFrame = !pForm->mo->showFrame;
                if(pForm->mo->showFrame == 1)
                    pForm->cb_frames->setChecked(TRUE);
                else
                    pForm->cb_frames->setChecked(FALSE);
            }
            else {
                pForm->mo->showFace = !pForm->mo->showFace;
                if(pForm->mo->showFace == 1)
                    pForm->cb_face->setChecked(TRUE);
                else
                    pForm->cb_face->setChecked(FALSE);
            }
                */
            //showFrame = !showFrame;
            break;
        //LEAVE GAP
        case Key_G:
            //leave_gap = !leave_gap;
            break;
        //DRAW-KNOT
        case Key_K:
            //draw_curve = !draw_curve;
            break;
        //LOCK-A&B    
        case Key_L:
             //if(e->state()&&ShiftButton)
             //    lock_a_b = !lock_a_b;
            break;
        //NORMALS
        case Key_N:
            //showNormals = !showNormals;
            break;
        //PRINT-COORDINATES
        case Key_P:
            break;
        //QUATERNION_MAP
        case Key_Q:
			fwin_pForm->calbak_Swap_Frames();
			//showQuatMap = !showQuatMap;
			//if(draw_curve==OBJKNOT || draw_curve==OBJSPLINE)
			//	oc->computeCurveQuaternionMap();
            break;            
        //RESET-APPLICATION
        case Key_R:
            this->resetApp();
            break;
        //FACES
        case Key_S:
            break;
        //TEXTURE
        case Key_T:
            /*if(e->state() && ShiftButton) {
                showTorus = !showTorus;
			}
            else
                showTexture = !showTexture;*/
            break;
        //QUATMAP-XYZ-PROJECTION
        case Key_W:
			fwin_pForm->mo->qset->whichQuatComponent = SHOWXYZ;
            fwin_pForm->rb_quat_comp_xyz->setChecked(TRUE);
			//whichQuatComponent = SHOWWYZ;
			//showViewXYZ	= 0;
			//showViewWXY	= 0;
			//showViewWXZ = 0;
			//showViewWYZ	= 1;
            break;
        //QUATMAP-WYZ-PROJECTION
        case Key_X:
			fwin_pForm->mo->qset->whichQuatComponent = SHOWWYZ;
            fwin_pForm->rb_quat_comp_wyz->setChecked(TRUE);
			//whichQuatComponent = SHOWXYZ;
			//showViewXYZ	= 1;
			//showViewWXY	= 0;
			//showViewWXZ = 0;
			//showViewWYZ	= 0;
            break;
		case Key_Y:
			fwin_pForm->mo->qset->whichQuatComponent = SHOWWXZ;
            fwin_pForm->rb_quat_comp_wxz->setChecked(TRUE);
			//whichQuatComponent = SHOWWXY;
			//showViewXYZ	= 0;
			//showViewWXY	= 1;
			//showViewWXZ = 0;
			//showViewWYZ	= 0;
			break;
        //BEZIER-SPLINE
		case Key_Z:
			fwin_pForm->mo->qset->whichQuatComponent = SHOWWXY;
            fwin_pForm->rb_quat_comp_wxy->setChecked(TRUE);
			//if(e->state() && ShiftButton) {
			//	currentSpline = BEZIER;
			//  whichSpline = BEZIER;
			//}
			//else {
			//	whichQuatComponent = SHOWWXZ;
			//	//showViewXYZ	= 0;
			//	//showViewWXY	= 0;
			//	//showViewWXZ = 1;
			//	//showViewWYZ	= 0;
			//}
            break;
            
        //FRENET-SERRET-FRAME
        case Key_1:
            //whichFrame = FRENETSERRET;
            break;
        //PARALLEL-TRANSPORT-FRAME    
        case Key_2:
            //whichFrame = PARALLELTRANSPORT;
            break;
        //QUATMAP-FRENET-SERRET-FRAME    
        case (Key_3):
            //whichQuatFrame = FRENETSERRET;
            break;
        //QUATMAP-PT-FRAME    
        case (Key_4):
            //whichQuatFrame = PARALLELTRANSPORT;
            break;
            
        case 27 :
        case Key_Escape:
            //if(mb.exec() == QMessageBox::Yes)
                fwin_pForm->exitApp();
            break;
            
        default:
            //fprintf(stderr,"Unhandled key: %c [%d] \n",key,key);
            break;
    }

	//buzzform();
    updateGL();
}
