/****************************************************************************/
/*  Copyright 2005, Trustees of Indiana University                          */
/*  Author: Sidharth Thakur                                                 */
/*  Last modified: 14 January 2006                                          */
/*  License available at: <belt-trick-* folder>/docs/downloads/License.txt  */
/****************************************************************************/
#include "OGLWidgetMain.h"
#include "VisibleForm.h"
#include <qimage.h>
#include <qmessagebox.h>
#include <qstring.h>

VisibleForm* pForm;
GLfloat diffuseMaterial[4] = {0.5,0.5,0.5,1.0};
char qstr[128];

void OGLWidgetMain::text(char* string) {
  char* p;
  for(p=string;*p;p++) {
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12,*p);
  }
}

void OGLWidgetMain::showOnScreenText() 
{
	double xpos=0.0;
	double ypos=0.0;
	double delta=0.07;
    
    glPushAttrib(GL_CURRENT_BIT | GL_LIGHTING_BIT);
    glDisable(GL_LIGHTING);
    
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
        glLoadIdentity();
        glMatrixMode(GL_PROJECTION);
        glPushMatrix();
        glLoadIdentity();
        gluOrtho2D(-1.2,1.2,-1.2,4.2);

        glMatrixMode(GL_MODELVIEW);
        glDisable(GL_LIGHTING);

        xpos=-1.0;
        ypos=-1.0;
        
		if(!hasFocus()){
			glColor3f(0.0f,0.0f,0.7f);
			sprintf(qstr,"Belt Trick ");
			glRasterPos2f(xpos,ypos);
			text(qstr);
			if(!hasFocus()) {
				xpos+=15.0*delta;
				glColor3f(0.7f,0.0f,0.0f);
				sprintf(qstr,"Click to focus...");
				glRasterPos2f(xpos,ypos);
				text(qstr);
			}
			xpos = -1.0;
		} 
		else {
			glColor3f(0.0f,0.0f,0.7f);
			sprintf(qstr,"Belt Trick Visualization");
			glRasterPos2f(xpos,ypos);
			text(qstr);
		}


        glMatrixMode(GL_PROJECTION);   
        glPopMatrix();
        glMatrixMode(GL_MODELVIEW);
    glPopMatrix();
    glPopAttrib();
}

OGLWidgetMain::OGLWidgetMain( QWidget* parent, const char* name, QWidget* parentForm)
: QGLWidget( parent, name)
{
	pForm = (VisibleForm*) parentForm;
	init();
}

OGLWidgetMain::~OGLWidgetMain() 
{
	pForm = NULL;
}

void OGLWidgetMain::initializeGL () 
{
    glClearColor(0.975,0.975,0.975,1.0);
    setupLights();
    m_viewScale = 1.0;
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glHint(GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
    
}

void OGLWidgetMain::init ( void )
{
	// workspace center
    X=0; Y=0; dX=0; dY=0;
    XC = 0.0; YC = 0.0; ZC = -10.0;
    Cx = 0.0; Cy = 0.0; Cz = 0.0;

    // mouse stuff
	mouseButtons	= 0;
    button			= 0;
    mouseModifiers	= 0;

	// viewport dimensions
    xMin			= -1.0;
    xMax			=  1.0;
    yMin			= -1.0;
    yMax			=  1.0;
    zNear			=  5.0;
    zFar			= 25.0;
	main_height		= this->height();
	main_width		= this->width();

        /*
    // workspace center
    X=0; Y=0; dX=0; dY=0;
    XC = 0.0; YC = 0.0; ZC = -10.0;
    Cx = 0.0; Cy = 0.0; Cz = 0.0;
        */
    dumpscreen = 0;
    
}

void OGLWidgetMain::initLighting ( void ) 
{

}

void OGLWidgetMain::setupLights (void) 
{
    static const GLfloat light_model_ambient[]	= {0.75f, 0.75f, 0.75f, 1.0f};
    static const GLfloat light0_ambient[]		= {0.75f, 0.75f, 0.75f, 1.0f};
    static const GLfloat light0_diffuse[]		= {1.0f, 1.0f, 1.0f, 1.0f};  
    static const GLfloat light0_specular[]		= {1.0f, 1.0f, 1.0f, 1.0f};
    static const GLfloat light0_shininess[]		= {500.0};
    static const GLfloat light0_direction[]		= {0.0f, 1.0f, 0.0f, 0.0f};    
    
    static const GLfloat light1_ambient[]		= {0.3f, 0.3f, 0.3f, 1.0f};
    static const GLfloat light1_diffuse[]		= {0.9f, 0.0f, 0.0f, 0.9f};  
    static const GLfloat light1_specular[]		= {1.0f, 1.0f, 1.0f, 1.0f};
    static const GLfloat light1_shininess[]		= {300.0};
    static const GLfloat light1_direction[]		= {0.0f, 10.0f, 10.0f, 0.0f};    
    
    //global_light0[0]=light0_direction[0];
    //global_light0[1]=light0_direction[1];
    //global_light0[2]=light0_direction[2];
    //global_light0[3]=light0_direction[3];
    
    glDepthFunc(GL_LEQUAL);
    glEnable(GL_DEPTH_TEST);
    
    glEnable(GL_LIGHTING);
    glEnable(GL_NORMALIZE);
    glShadeModel(GL_SMOOTH);
    
    glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_FALSE);
    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);    
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, light_model_ambient);

    glLightfv(GL_LIGHT0, GL_DIFFUSE,   light0_diffuse);
    glLightfv(GL_LIGHT0, GL_POSITION,  light0_direction);
    glLightfv(GL_LIGHT0, GL_AMBIENT,   light0_ambient);
    glLightfv(GL_LIGHT0, GL_SPECULAR,  light0_specular);
    glLightfv(GL_LIGHT0, GL_SHININESS, light0_shininess);
    glEnable(GL_LIGHT0);     

    glLightfv(GL_LIGHT1, GL_DIFFUSE,  light1_diffuse);
    glLightfv(GL_LIGHT1, GL_POSITION, light1_direction);
    glLightfv(GL_LIGHT1, GL_AMBIENT,  light1_ambient);
    glLightfv(GL_LIGHT1, GL_SPECULAR, light1_specular);
    glLightfv(GL_LIGHT1, GL_SHININESS,light1_shininess);
	
    glEnable(GL_LIGHT0);

}


void OGLWidgetMain::paintGL ()
{
	double factor = 25.0;
	double factor2= 3.75;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glPushAttrib( GL_ALL_ATTRIB_BITS );
		glPushMatrix();
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		glEnable(GL_NORMALIZE);
		glDisable(GL_LIGHTING);
		glEnable(GL_COLOR_MATERIAL);
		//glDisable(GL_CULL_FACE);
		
		glBegin(GL_QUADS); //BACK
			glColor3f(0.9,0.9,0.9);
			glNormal3f(0.0f,0.0f,1.0f);
			glVertex3f(-factor2, -factor2, -factor);
			glVertex3f(+factor2, -factor2, -factor);
			glVertex3f(+factor2, +factor2, -factor);
			glVertex3f(-factor2, +factor2, -factor);
		glEnd();
		glBegin(GL_QUADS); //LEFT
			glColor3f(0.75,0.75,0.75);
			glNormal3f(1.0f,0.0f,0.0f);
			glVertex3f(-factor2, +factor2, -factor);
			glVertex3f(-factor2, +factor2, +factor);
			glVertex3f(-factor2, -factor2, +factor);
			glVertex3f(-factor2, -factor2, -factor);
		glEnd();
		glBegin(GL_QUADS); //RIGHT
			glColor3f(0.75,0.75,0.75);
			glNormal3f(-1.0f,0.0f,0.0f);
			glVertex3f(+factor2, +factor2, -factor);
			glVertex3f(+factor2, -factor2, -factor);
			glVertex3f(+factor2, -factor2, +factor);
			glVertex3f(+factor2, +factor2, +factor);
		glEnd();
		glBegin(GL_QUADS); //TOP
			glColor3f(0.6,0.6,0.6);
			glNormal3f(0.0f,1.0f,0.0f);
			glVertex3f(+factor2, +factor2, +factor);
			glVertex3f(+factor2, +factor2, -factor);
			glVertex3f(-factor2, +factor2, -factor);
			glVertex3f(-factor2, +factor2, +factor);
		glEnd();
		glPopMatrix();
	glPopAttrib();

	glPushMatrix();
		glTranslatef(Cx, Cy, 0);
		glScalef(m_viewScale, m_viewScale, m_viewScale);
		pForm->mo->display();
	glPopMatrix();
	
    showOnScreenText();    
}

void OGLWidgetMain::resetApp ( void )
{
    m_viewScale = 1.0;
    
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glFrustum(xMin, xMax, yMin, yMax, zNear, zFar);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glScalef(m_viewScale, m_viewScale, m_viewScale);//@ scale factor
    glTranslatef(XC,YC,ZC);
}

void  OGLWidgetMain::resizeGL( int wd, int ht )
{
    int w = wd;
    int h = ht;

    glViewport(0,0,w,h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    glFrustum(xMin, xMax, yMin, yMax, zNear, zFar);
    /* Viewing location of camera */
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glTranslatef(XC,YC,ZC);
    
}

void OGLWidgetMain::mousePressEvent ( QMouseEvent *e )
{
    QMouseEvent *m = e;
    QPixmap pixmap;
    int x = m->x();
    int y = m->y();

    grabKeyboard();
    
    X = x;
    Y =  main_height -y - 1;
    switch (m->button()){
        case  LeftButton:
            button = 0x1; /* our Left   bit code */
            pForm->lb_status->setText(QString("Rotate model: Drag mouse while pressing Left-button"));
            pixmap = QPixmap::fromMimeSource("images/icon-action-drag-rotate.png");
            pForm->pix_status->setPixmap(pixmap);
            break;
        case MidButton:
            button = 0x2; /* our Middle bit code */
            pForm->lb_status->setText(QString("Slide Tracker: Drag mouse while pressing Middle-button"));
            pixmap = QPixmap::fromMimeSource("images/icon-action-drag-tracker.png");
            pForm->pix_status->setPixmap(pixmap);
            break;
        case RightButton:
            button = 0x4; /* our Right  bit code */
            pForm->lb_status->setText(QString("Zoom model: Drag mouse while pressing Right-button"));
            pixmap = QPixmap::fromMimeSource("images/icon-action-drag-zoom.png");
            pForm->pix_status->setPixmap(pixmap);
            break;
        default:
            break;
    }

    mouseButtons |= button;
    
    //Check if a vertex is lit
    if(button == 0x1){
        if(litVertex == -1){
			//if(draw_curve==OBJSPLINE) {
   //             //if not, add a new vertex
   //             printf("mouse clicked at (%d,%d)\n",X,Y);
   //             addPoint(X,Y);
   //         }
        }
        else{
			//printf("litVertex = %d\n",litVertex);
            movingPoint = TRUE;
        }   
    }

    updateGL();
}

void OGLWidgetMain::mouseReleaseEvent ( QMouseEvent *e )
{
	QMouseEvent *evnt = e;
    QPixmap pixmap;
	mouseButtons &= (~(button));
    if(button == 0x1){
        movingPoint = FALSE;
    }

    pForm->lb_status->setText(QString(""));
    pixmap = QPixmap::fromMimeSource("images/icon-empty.png");
    pForm->pix_status->setPixmap(pixmap);

	evnt = NULL;
}

void OGLWidgetMain::moveTransformedCoordinate(float x, float y)
{

    GLint viewport[4],winHeight;
    GLdouble mvmatrix[16], projmatrix[16];
    GLdouble wx,wy,wz, objx, objy, objz;
    
    
    glGetDoublev(GL_MODELVIEW_MATRIX, mvmatrix);
    glGetDoublev(GL_PROJECTION_MATRIX, projmatrix);
    glGetIntegerv(GL_VIEWPORT,viewport);
    winHeight = viewport[3];
    
    if(gluProject(xCoord[litVertex], yCoord[litVertex], zCoord[litVertex],
                  mvmatrix, projmatrix, viewport, &wx,&wy,&wz) == GL_FALSE){
        printf("gluProject() didn't work in moveTransformedCooordinates\n");
    }
    
    if(gluUnProject((GLdouble)x, 
                    (GLdouble)y,wz,mvmatrix,projmatrix,viewport, &objx,&objy,&objz) == GL_FALSE){
        printf("gluUnProject() didn't work in moveTransformedCooordinates\n");
    }
    
    xCoord[litVertex]=objx;
    yCoord[litVertex]=objy;
	//buzzform();
}
void OGLWidgetMain::mouseMoveEvent ( QMouseEvent *e )
{
    QMouseEvent *m = e;
    int		x = m->x();
    int		y = m->y();
    double	deltaY	= 0.0;
    double	deltaX	= 0.0;
    int		nHatX	= 0;
    int		nHatY	= 0;
    double	thetaRad = 0.0;
    double	thetaDeg = 0.0;
    double	radius	 = 150.0;
    double	drSquared= 0.0;
    int		deltaX_trackerpos = 0;
	int		scale_tracker = pForm->mo->bspheres/75;
	if(scale_tracker <= 1) 
		scale_tracker = 1;
	float	modelviewMatrix[16];
	

    
    //@ scaling
    double	y1=0.0;
	double	y2=0.0;
    
    xBegin = X;
    yBegin = Y;
    startY = Y;
    X = x;
    Y =  main_height - y - 1;	/* Invert to right-handed system */

    //@ scaling objects
    y1 = (double)yBegin;
    y2 = (double)Y;

    xNext = X;
    yNext = Y;

   // Passive motion callback is simulate here with the
   // boolean movingPoint
    //if((enable_picking) && (draw_curve==OBJSPLINE) && (!movingPoint)) {
    //    mousePickVertex(x,y);
    //}
    
    //case 0x1:  /* Middle Mouse Button   */
    if(m->state()==MidButton) {
        
		if(mouseModifiers != GLUT_ACTIVE_SHIFT){
			pForm->mo->flag_do_not_compute = 1;
			//moveTransformedCoordinate(X,Y);
			deltaX_trackerpos = xNext - xBegin;
			if(deltaX_trackerpos > 1)
				deltaX_trackerpos = 2;
			if(deltaX_trackerpos < -1)
				deltaX_trackerpos = -2;

			pForm->mo->btrackerpos += scale_tracker*deltaX_trackerpos;
			if(pForm->mo->btrackerpos <0)
				pForm->mo->btrackerpos = 0;
			else if( pForm->mo->btrackerpos > pForm->mo->bspheres-1)
				pForm->mo->btrackerpos = pForm->mo->bspheres-1;

			pForm->notifyOGLWidgets();
		}
		else{
			//shift is pressed so move in Z
			deltaY = Y - (int)startY;
			zCoord[litVertex] = zCoord[litVertex] - (float)deltaY/20.0;
			if(zCoord[litVertex] > zNear){
				zCoord[litVertex] = zNear;
			}
			if(zCoord[litVertex] < (zNear - (zFar - zNear))){
				zCoord[litVertex] =(zNear - (zFar - zNear)) ;
			}
		}
    }
    
    //case 0x2:  /* Left Mouse Button - ROTATE OBJECT */
    if(m->state()==LeftButton){

            deltaX = xNext - xBegin;
            deltaY = yNext - yBegin;
            nHatX = (int)-deltaY;
            nHatY = (int)deltaX;
            //get the amount of rotation
            drSquared = pow(deltaX,2) + pow(deltaY,2);
            thetaRad = acos(radius/(sqrt(pow(radius,2) + drSquared)));
            thetaDeg = (thetaRad * 180) / M_PI;
            //translate to center
            glGetFloatv(GL_MODELVIEW_MATRIX,modelviewMatrix);
            glMatrixMode(GL_MODELVIEW);
            glLoadIdentity();
            glTranslatef(XC,YC,ZC);
            glTranslatef(Cx, Cy, 0);
            
            glRotatef(thetaDeg,nHatX,nHatY,0);
            //tranlsate back
            glTranslatef(-Cx, -Cy, 0);
            glTranslatef(-XC,-YC,-ZC);
            glMultMatrixf(modelviewMatrix);

        
    }
    
    //case 0x4:  /* Right Mouse Button  */
    //translating
    if( (m->state() & RightButton) && (m->state() & ShiftButton)) {
        
            glGetFloatv(GL_MODELVIEW_MATRIX,modelviewMatrix);
            glMatrixMode(GL_MODELVIEW);
            glLoadIdentity();
            glTranslatef(((float)xNext-xBegin)/((float)main_width/2.0),
            ((float)yNext-yBegin)/((float)main_height/2.0),0);
            glMultMatrixf(modelviewMatrix);

        
        Cx = Cx + ((float)xNext-xBegin)/((float)main_width/2.0);
        Cy = Cy + ((float)yNext-yBegin)/((float)main_height/2.0);
    }
    else  if((m->state() & RightButton) && !(m->state() & ShiftButton))//scaling
    {
        m_viewScale *= 1 + (y2 - y1)/((double)main_height*10.0);
        if(m_viewScale<0.5)
            m_viewScale=0.5;
        if(m_viewScale>5.0)
            m_viewScale=5.0;
    }        
    updateGL();
	pForm->mo->flag_do_not_compute = 0;
}

void OGLWidgetMain::keyPressEvent ( QKeyEvent *e )
{
	QMessageBox mb( "Exiting...",
                    "Really quit?",
                    QMessageBox::NoIcon,
                    QMessageBox::Yes | QMessageBox::Default,
                    QMessageBox::No,
					QMessageBox::NoButton );
    //modkey = e->key();
    switch(e->key()) {

        //CLEAR ALL
        case Key_Space:
            //myform->clearAll();
            break;
        //HELP    
        case Key_H:
			pForm->calbak_Help();
			break;
        //AXES
        case Key_A:
			if(e->state() && ShiftButton) {
				pForm->mo->flag_blend = !pForm->mo->flag_blend;
			}
			else {
				pForm->mo->showAxes = !pForm->mo->showAxes;
				if(pForm->mo->showAxes == 1)
					pForm->cb_axes->setChecked(TRUE);
				else
					pForm->cb_axes->setChecked(FALSE);
			}
            break;
        //B-SPLINE
        case Key_B:
			pForm->mo->inp_chkbox_belt_spheres_wire = !pForm->mo->inp_chkbox_belt_spheres_wire;
			if(pForm->mo->inp_chkbox_belt_spheres_wire) {
				pForm->slider_belt_control_spheres_number->setEnabled(TRUE);
				pForm->cb_belt_bounding_spheres->setChecked(TRUE);
			}
			else {
				pForm->slider_belt_control_spheres_number->setEnabled(FALSE);
				pForm->cb_belt_bounding_spheres->setChecked(FALSE);
			}
			break;
        //CATMULL-ROM SPLINE    
        case Key_C:
            pForm->mo->showSphere = !pForm->mo->showSphere;
            break;
        //EDGES
        case Key_E: 
			if(pForm->mo->flag_track_belt == -1) {
				pForm->mo->showEdge = !pForm->mo->showEdge;
				if(pForm->mo->showEdge == 1)
					pForm->cb_edge->setChecked(TRUE);
				else
					pForm->cb_edge->setChecked(FALSE);
			}
            break;
        //FRAME
        case Key_F:
            if(e->state() && ShiftButton) {
                pForm->mo->showFrame = !pForm->mo->showFrame;
                if(pForm->mo->showFrame == 1)
                    pForm->cb_frames->setChecked(TRUE);
                else
                    pForm->cb_frames->setChecked(FALSE);
            }
            else {
                pForm->mo->showFace = !pForm->mo->showFace;
                if(pForm->mo->showFace == 1)
                    pForm->cb_face->setChecked(TRUE);
                else
                    pForm->cb_face->setChecked(FALSE);
            }
            
            //showFrame = !showFrame;
            break;
        //LEAVE GAP
        case Key_G:
            //leave_gap = !leave_gap;
            break;
		case Key_I:
			pForm->calbak_Instructions();

        //DRAW-KNOT
        case Key_K:
            //draw_curve = !draw_curve;
            break;
        //LOCK-A&B    
        case Key_L:
             //if(e->state()&&ShiftButton)
             //    lock_a_b = !lock_a_b;
            break;
        //NORMALS
        case Key_N:
            pForm->mo->inp_chkbox_belt_show_numbers = !pForm->mo->inp_chkbox_belt_show_numbers;
			if(pForm->mo->inp_chkbox_belt_show_numbers)
				pForm->cb_belt_numbers->setChecked(TRUE);
			else
				pForm->cb_belt_numbers->setChecked(FALSE);
            break;
		case Key_O:
			pForm->mo->bhiderest = !pForm->mo->bhiderest;
			if(pForm->mo->bhiderest)
				pForm->cb_belt_multiple_hide_rest->setChecked(TRUE);
			else
				pForm->cb_belt_multiple_hide_rest->setChecked(FALSE);
			break;
        //PRINT-COORDINATES
        case Key_P:
            break;
        //QUATERNION_MAP
        case Key_Q:
			pForm->calbak_Swap_Frames();
			//showQuatMap = !showQuatMap;
			//if(draw_curve==OBJKNOT || draw_curve==OBJSPLINE)
			//	oc->computeCurveQuaternionMap();
            break;            
        //RESET-APPLICATION
        case Key_R:
            this->resetApp();
            break;
        //FACES
        case Key_S:
			////pForm->mo->bspine = !pForm->mo->bspine;
			////if(pForm->mo->bspine)
			////	pForm->cb_spine->setChecked(TRUE);
			////else
			////	pForm->cb_spine->setChecked(FALSE);
            break;
        //TEXTURE
        case Key_T:
			pForm->mo->showTracker = !pForm->mo->showTracker;
			if(pForm->mo->showTracker)
				pForm->cb_tracker->setChecked(TRUE);
			else
				pForm->cb_tracker->setChecked(FALSE);
            break;
        //QUATMAP-XYZ-PROJECTION
        case Key_W:
			//whichQuatComponent = SHOWWYZ;
			//showViewXYZ	= 0;
			//showViewWXY	= 0;
			//showViewWXZ = 0;
			//showViewWYZ	= 1;
            break;
        //QUATMAP-WYZ-PROJECTION
        case Key_X:
			//whichQuatComponent = SHOWXYZ;
			//showViewXYZ	= 1;
			//showViewWXY	= 0;
			//showViewWXZ = 0;
			//showViewWYZ	= 0;
            break;
		case Key_Y:
			//whichQuatComponent = SHOWWXY;
			//showViewXYZ	= 0;
			//showViewWXY	= 1;
			//showViewWXZ = 0;
			//showViewWYZ	= 0;
			break;
        //BEZIER-SPLINE
		case Key_Z:
			//if(e->state() && ShiftButton) {
			//	currentSpline = BEZIER;
			//  whichSpline = BEZIER;
			//}
			//else {
			//	whichQuatComponent = SHOWWXZ;
			//	//showViewXYZ	= 0;
			//	//showViewWXY	= 0;
			//	//showViewWXZ = 1;
			//	//showViewWYZ	= 0;
			//}
            break;
            
        case Key_1:
            pForm->mo->bpattern = BELT_PATTERN_DOUBLE_SIDED;
			//whichFrame = PARALLELTRANSPORT;
            break;
        case Key_2:
			pForm->mo->bpattern = BELT_PATTERN_SEGMENTS;
            //whichFrame = FRENETSERRET;
            break;
        case (Key_3):
            //whichQuatFrame = FRENETSERRET;
            break;
        case (Key_4):
            //whichQuatFrame = PARALLELTRANSPORT;
            break;
            
        case 27 :
        case Key_Escape:
            if(mb.exec() == QMessageBox::Yes)
                pForm->exitApp();
            break;
            
        default:
            //fprintf(stderr,"Unhandled key: %c [%d] \n",key,key);
            break;
    }

	//buzzform();
    updateGL();
}

int OGLWidgetMain::saveToBMP ( int width, int height )
{
	static int	counter = 1;
	char		fname[32];

    int l_width = width;
    int l_height= height;
    l_width = 0;
    l_height= 0;
	//# Make GL Context current
	makeCurrent();

	//# Copy from OpenGL screen buffer
	QImage *tempImage = new QImage( grabFrameBuffer() );

	if ( !tempImage ) {
		fprintf(stderr,"Error! Failed to allocate memory for image\n");
		return false;
	}

	//# Create filename
	if(pForm->mo->inp_chkbox_belt_type==BELT360) {
		if(pForm->mo->bpattern == BELT_PATTERN_SEGMENTS)
			sprintf(fname,"./snapshots/striped_360_%01d.bmp",counter);
		else
			sprintf(fname,"./snapshots/plain_360_%01d.bmp",counter);
	}
	else {
		if(pForm->mo->bpattern == BELT_PATTERN_SEGMENTS)
			sprintf(fname,"./snapshots/striped_720_%01d.bmp",counter);
		else
			sprintf(fname,"./snapshots/plain_720_%01d.bmp",counter);
	}

	if ( !tempImage->save( QString(fname), "BMP") ) {
		fprintf(stderr,"Error! Could not save BMP file...\n");
		return false;
	}
	else {
		printf("Saved! Writing out file: %s\n",fname);
		counter++;
	}
	 
	//# Clean memory
	delete tempImage;

	return true;
}

int OGLWidgetMain::DumpScreen( int width,int height )
{
    int			i=0, j=0;
    FILE		*fptr;
    static int		counter = 1;
    char		fname[32];
    unsigned char	*image;

        //fprintf(stderr,"width = %d, height = %d\n",width, height);

    /* Allocate our buffer for the image */
    if ((image = (unsigned char*) malloc(3*width*height*sizeof(char))) == NULL) {
       fprintf(stderr,"WindowDump - Failed to allocate memory for image\n");
       return(FALSE);
    }

    
    /* Open the file */
    if(pForm->mo->inp_chkbox_belt_type==BELT360) {
        if(pForm->mo->bpattern == BELT_PATTERN_SEGMENTS)
            sprintf(fname,"./snapshots2/striped_360_%01d.ppm",counter);
        else
            sprintf(fname,"./snapshots2/plain_360_%01d.ppm",counter);
    }
    else {
        if(pForm->mo->bpattern == BELT_PATTERN_SEGMENTS)
            sprintf(fname,"./snapshots2/striped_720_%01d.ppm",counter);
        else
            sprintf(fname,"./snapshots2/plain_720_%01d.ppm",counter);
    }
    
    
    if ((fptr = fopen(fname,"w")) == NULL) {
       fprintf(stderr,"WindowDump - Failed to open file for window dump\n");
       return(FALSE);
    }

    /* Copy the image into our buffer */
    glReadBuffer(GL_BACK);
    glReadPixels(0,0,width,height,GL_RGB,GL_UNSIGNED_BYTE,image);

    /* Write the PPM file */
    
    fprintf(fptr,"P6\n%d %d\n255\n",width,height);
    for (j=height-1;j>=0;j--) {
       for (i=0;i<width;i++) {

               /*if(image[3*j*width+3*i]!=255)
                 printf("%d\n",image[3*j*width+3*i]);*/
           
          fputc(image[3*j*width+3*i+0],fptr);
          fputc(image[3*j*width+3*i+1],fptr);
          fputc(image[3*j*width+3*i+2],fptr);
       }
    }
    fclose(fptr);
    printf("wrote file... ./snapshots/-%01d.ppm",counter);
    

  free(image);
  counter++;
  return(TRUE);

}
