/****************************************************************************/
/*  Copyright 2005, Trustees of Indiana University                          */
/*  Author: Sidharth Thakur                                                 */
/*  Last modified: 14 January 2006                                          */
/*  License available at: <belt-trick-* folder>/docs/downloads/License.txt  */
/****************************************************************************/
#ifndef _QUATERNION_H_
#define _QUATERNION_H_

///////////////////////////////////////////////////////////////////////////////
//
// $Header:  quatenrnion.h$
//
// NAME: none
//
// BASE  CLASSES: none
//  
// PURPOSE: Global include for quaternion data-type
//
// AUTHOR: Sidharth Thakur
//
///////////////////////////////////////////////////////////////////////////////

#include <math.h>

typedef struct {
        
 double r;
    double i;
    double j;
    double k;
}quaternion;


/* QUATERNION UTILITIES */
void quatBuildQuaternion		( double theta, double i, double j, double k, quaternion *q );
void quatBuildQuaternion		( double theta, double* vector, quaternion *q );
void quatMakeUnitQuaternion		( quaternion *q, double r, double i, double j, double k );
void quatGenerate				( quaternion *q, double theta, double *v);
void quatConjugate				( quaternion *q ); 
double quatQuatDistance			( quaternion *q1, quaternion *q2 );
double quatQuatDotProduct		( quaternion *q1, quaternion *q2 );
void quatQuatProduct			( quaternion *q1, quaternion *q2, quaternion *q3);
void quatToRotationMatrix		( quaternion *q, double *mat );
void quatToRotationMatrixf		( quaternion *q, float *mat );
void quatFromRotationMatrix		( double *mat, quaternion *q );
double qNorm					( quaternion* q );
double qNorm2					( quaternion q );
void quatMakeQuaternion			( quaternion *q, double r, double i, double j, double k );
void quatNormalize				( quaternion *q );
void quatCopy					( const quaternion *q1, quaternion *q2 );
void ExtractQuaternionFromMat	( double m[16], quaternion *q );

#endif
